"""
 Copyright (C) 2014 Maruf Maniruzzaman
 Website: http://cosmosframework.com
 Author: Maruf Maniruzzaman
 License :: OSI Approved :: MIT License
"""
from argparse import _ActionsContainer
import tornado
from tornado import gen
import collections
import uuid
import logging
from cosmos.rbac.cache import RoleCache
from cosmos.rbac.object import *

# TODO: Remove global usage

class RbacService():
    def __init__(self, *args, **kwargs):
        self.role_cache = kwargs.get("role_cache", RoleCache())

    def get_role_group(self, sid):
        """
        Find and return RoleGroup object by sid
        :param sid: Role group sid
        """
        role_groups = self.role_cache.get_role_groups()

        for group in role_groups:
            if sid == group.sid:
                return group

        return None

    #TODO: Return expanded role groups instead of taking out parameter
    def expand_role_group(self, role_group, expanded_roles):
        """
        Expand the provided role_group recursively and store results into a list.

        :param role_group: role group to expand
        :param expanded_roles: list to store expanded roles.
        """

        assert isinstance(role_group, RoleGroup)
        assert isinstance(expanded_roles, collections.Iterable)

        for role_sid in role_group.role_sids:
            role = self.role_cache.get_role(role_sid)
            if role:
                expanded_roles.append(role_sid)
            else:
                child_role_group = self.get_role_group(role_sid)
                if child_role_group:
                    self.expand_role_group(child_role_group, expanded_roles)
                else:
                    logging.error("Undefined Role {} in Group {}".format(role_sid, role_group.sid))

    def get_all_roles(self):
        return self.role_cache.get_roles()

    def get_roles(self, user):
        active_roles = self.role_cache.get_roles()
        active_role_groups = self.role_cache.get_role_groups()

        roles = []

        if not user:
            user_role_sids = [ANONYMOUS_USER_ROLE_SID]
        else:
            user_role_sids = user.get("roles", [ANONYMOUS_USER_ROLE_SID, LOGGED_IN_USER_ROLE_SID])
            if LOGGED_IN_USER_ROLE_SID not in user_role_sids:
                user_role_sids.append(LOGGED_IN_USER_ROLE_SID)

        if ANONYMOUS_USER_ROLE_SID not in user_role_sids:
            user_role_sids.append(ANONYMOUS_USER_ROLE_SID)

        for role in active_roles:
            if role.sid in user_role_sids:
                roles.append(role)

        for role_group in active_role_groups:
            if role_group.sid in user_role_sids:
                expanded_roles = []
                self.expand_role_group(role_group, expanded_roles)
                for role_sid in expanded_roles:
                    role = self.role_cache.get_role(role_sid)
                    roles.append(role)

        return roles

    def has_owner_access(self, role, object_name, properties, access):
        assert isinstance(role, Role)
        assert isinstance(properties, collections.Iterable)

        required_properties = properties[:]

        for role_item in role.role_items:
            assert isinstance(role_item, RoleItem)

            if (role_item.object_name == ALLOW_ALL_OBJECT_NAME or role_item.object_name == object_name) \
                    and access in role_item.owner_access:

                #Check for blanket access
                if ALLOW_ALL_PROPERTY_NAME == role_item.property_name:
                    return True

                #We require blanket approval (*) when properties are not specified
                if len(properties) == 0:
                    continue

                if role_item.property_name in required_properties:
                    required_properties.remove(role_item.property_name)

        return len(properties) > 0 and len(required_properties) == 0

    def has_access(self, role, object_name, properties, access):
        assert isinstance(role, Role)
        assert isinstance(properties, collections.Iterable)

        required_properties = properties[:]

        for role_item in role.role_items:
            assert isinstance(role_item, RoleItem)

            if (role_item.object_name == ALLOW_ALL_OBJECT_NAME or role_item.object_name == object_name) \
                    and access in role_item.access:

                #Check for blanket access
                if ALLOW_ALL_PROPERTY_NAME == role_item.property_name:
                    return True

                #We require blanket approval (*) when properties are not specified
                if len(properties) == 0:
                    continue

                if role_item.property_name in required_properties:
                    required_properties.remove(role_item.property_name)

        return len(properties) > 0 and len(required_properties) == 0

    def get_role_object(self, role_def):
        return self.role_cache.get_role_object(role_def)

    def update_role_cache(self, role_defn):
        self.role_cache.update_role_cache(role_defn)

    def update_role_group_cache(self, role_group_defn):
        self.role_cache.update_role_group_cache(role_group_defn)

    def clear_non_system_roles(self):
        self.role_cache.clear_non_system_roles()

    def clear_non_system_role_groups(self):
        self.role_cache.clear_non_system_role_groups()


sample_role = {
    "name": "Administrator",
    "role_items": [
        {
            "access": [
                "INSERT",
                "READ",
                "WRITE"
            ],
            "object_name": "testservice",
            "property_name": "name",
            "type": "object.RoleItem"
        },
        {
            "access": [
                "INSERT",
                "READ",
                "WRITE"
            ],
            "object_name": "testservice",
            "property_name": "address",
            "type": "object.RoleItem"
        },
        {
            "access": [
                "INSERT",
                "READ",
                "WRITE"
            ],
            "object_name": "testservice",
            "property_name": "*",
            "type": "object.RoleItem"
        },
        {
            "access": [
                "DELETE"
            ],
            "object_name": "testservice",
            "property_name": "*",
            "type": "object.RoleItem"
        }
    ],
    "type": "object.Role",
    "sid": "43425097-e630-41ea-88eb-17b339339707"
}

sample_role_group = {
    "name": 'TestGroup',
    "sid": '3222c945-48eb-493f-9388-9f06292b27d3',
    "role_sids": [
        "43425097-e630-41ea-88eb-17b339339707",
        "3222c945-48eb-493f-9388-9f06292b27d2"
    ],
    "type": "object.RoleGroup"
}

if __name__ == "__main__":
    serv = RbacService()

    for role in serv.get_all_roles():
        print (role.to_JSON())

    role = serv.get_role_object(sample_role)
    role_json = role.to_JSON()
    role_def = json.loads(role_json)
    role2 = serv.get_role_object(sample_role)
    print (role2.to_JSON())
    serv.update_role_cache(sample_role)
    serv.update_role_group_cache(sample_role_group)
    group = serv.get_role_group(sample_role_group["sid"])
    expand_role_group = []
    serv.expand_role_group(group, expand_role_group)

    assert len(expand_role_group) == 2

    assert serv.has_access(role, "testservice", ["name"], AccessType.READ)
    assert serv.has_access(role, "testservice", ["name"], AccessType.UPDATE)
    assert serv.has_access(role, "testservice", ["address"], AccessType.READ)


"""
 Copyright (C) 2014 Maruf Maniruzzaman
 Website: http://cosmosframework.com
 Author: Maruf Maniruzzaman
 License :: OSI Approved :: MIT License
"""

import os
import subprocess

from cosmos.admin.samples import barebonedef, simpledef, angulardef, angularbasicdef, adminpaneldef

def new_project(path, type=None):
    file_data_list = barebonedef.file_data_list

    if type == None:
        type = "adminpanel"

    if type == "angular":
        file_data_list.extend(angulardef.file_data_list)
    elif type == "angularbasic":
        file_data_list.extend(angularbasicdef.file_data_list)
    elif type == "adminpanel":
        file_data_list.extend(adminpaneldef.file_data_list)
    elif type == "simple":
        file_data_list.extend(simpledef.file_data_list)

    for file_data in file_data_list:
        filename = file_data["name"]
        data = file_data["data"]
        if filename[0]=='/':
            filename = filename[1:]
        file_path = os.path.join(path, filename)
        dir_name = os.path.dirname(file_path)
        if not os.path.exists(dir_name):
            os.makedirs(dir_name)
        with open(file_path, 'w') as content_file:
            content_file.write(data)

    if type == "angular":
        print("-----------Cloning angular seed project--------------\n")
        try:
            subprocess.check_call(['git', "clone", "https://github.com/angular/angular-seed.git"])
            print('----------- You should run "npm install" from angular-seed directory now -------------\n')
        except subprocess.CalledProcessError:
            print ("Clone failed (is git installed?). You may try to clone manually using 'git clone https://github.com/angular/angular-seed.git'")
    elif type == "adminpanel":
        print("---------------------------------------------------------------------------------------\n")
        print('----------- You should run "bower install" from the project directory now -------------\n')
        print('Bower or any other npm package is NOT required on production. Only for development purpose.\n')


if __name__ == "__main__":
    path = os.getcwd()
    new_project(path)




"""
 Copyright (C) 2014 Maruf Maniruzzaman
 Website: http://cosmosframework.com
 Author: Maruf Maniruzzaman
 License :: OSI Approved :: MIT License
"""

from cosmos.service import requesthandler

import tornado.ioloop
import tornado.web
import tornado.options
import tornado.template
import tornado.websocket

from cosmos.service.utils import MongoObjectJSONEncoder
from cosmos.dataservice.objectservice import *


class GridFSServiceHandler(requesthandler.RequestHandler):

    @gen.coroutine
    def get(self, object_path):
        params = object_path.split('/')
        params = list(filter(len, params))
        if len(params) < 1 or len(params)>2:
            raise tornado.web.HTTPError(404, "Not found")

        object_name = params[0]

        if not object_name:
            raise tornado.web.HTTPError(404, "Not found")

        object_full_name = object_name

        id = None
        if len(params)==2:
            id = params[1]

        obj_serv = self.settings['object_service']

        preprocessor_list = obj_serv.get_operation_preprocessor(object_full_name, AccessType.READ)
        for preprocessor in preprocessor_list:
            yield  preprocessor(obj_serv, object_full_name, id, AccessType.READ)

        result = None
        if id and len(id)>0:
            promise = obj_serv.load_file(self.current_user, object_full_name, id)
            result = yield promise

            self.write(result.get("body"))
            content_type = result.get("content_type")
            if content_type:
                self.set_header("Content-Type", content_type)
        else:
            list_allowed = self.settings.get("directory_listing_allowed")
            if list_allowed:
                promise = obj_serv.list_file(self.current_user, object_full_name)
                result = yield promise
                data = {"_d": MongoObjectJSONEncoder().encode(result), "_cosmos_service_array_result_": True};

                self.write(json.dumps(data))
            else:
                raise tornado.web.HTTPError(403, "Directory listing forbidden.")
                """
                self.write('''
                <form enctype="multipart/form-data" method="POST">
                    File tu upload: <input name="uploadedfile" type="file" /><br />
                    <input type="submit" value="Upload File" />
                </form>''')
                """

        post_processor_list = obj_serv.get_operation_postprocessor(object_full_name, AccessType.READ)
        for post_processor in post_processor_list:
            yield post_processor(obj_serv, object_full_name, result, AccessType.READ)

        self.finish()

    @gen.coroutine
    def post(self, object_path):
        params = object_path.split('/')
        params = list(filter(len, params))
        object_name = params[0]

        object_full_name = object_name
        files = self.request.files["uploadedfile"]

        obj_serv = self.settings['object_service']

        result_list = []
        for file in files:
            preprocessor_list = obj_serv.get_operation_preprocessor(object_full_name, AccessType.INSERT)
            for preprocessor in preprocessor_list:
                yield preprocessor(obj_serv, object_full_name, file, AccessType.INSERT)

            promise = obj_serv.save_file(self.current_user, object_full_name, file)
            result = yield promise

            result_list.append(result)

            post_processor_list = obj_serv.get_operation_postprocessor(object_full_name, AccessType.INSERT)
            for post_processor in post_processor_list:
                yield  post_processor(obj_serv, object_full_name, result, AccessType.INSERT)

        data = {"_d": MongoObjectJSONEncoder().encode(result_list), "_cosmos_service_array_result_": True};

        self.write(data)
        self.finish()

    @gen.coroutine
    def put(self, object_path):
        params = object_path.split('/')
        params = list(filter(len, params))
        object_name = params[0]

        if not object_name or len(object_name) < 1:
            raise tornado.web.HTTPError(404, "Not found")

        object_full_name = object_name

        file_id = None
        if len(params) == 2:
            file_id = params[1]
        else:
            raise tornado.web.HTTPError(400, "File id required")


        if not file_id or len(file_id)<1:
            raise tornado.web.HTTPError(400, "File id required")

        obj_serv = self.settings['object_service']

        preprocessor_list = obj_serv.get_operation_preprocessor(object_full_name, AccessType.UPDATE)
        for preprocessor in preprocessor_list:
            yield preprocessor(object_full_name, None, AccessType.UPDATE)

        files = self.request.files["uploadedfile"]

        if (not files) or len(files) != 1:
            raise tornado.web.HTTPError(400, "Exactly one file should be uploaded to update a file.")

        uploaded_file = files[0]

        promise = obj_serv.save_file(self.current_user, object_full_name, uploaded_file, file_id)
        result = yield promise

        post_processor_list = obj_serv.get_operation_postprocessor(object_full_name, AccessType.UPDATE)
        for post_processor in post_processor_list:
            yield post_processor(obj_serv, object_full_name, None, AccessType.UPDATE)

        self.write(MongoObjectJSONEncoder().encode(result))
        self.finish()

    @gen.coroutine
    def delete(self, object_path):
        params = object_path.split('/')
        params = list(filter(len, params))
        object_name = params[0]

        if not object_name or len(object_name) < 1:
            raise tornado.web.HTTPError(404, "Not found")

        object_full_name = object_name

        file_id = None
        if len(params) == 2:
            file_id = params[1]
        else:
            raise tornado.web.HTTPError(400, "File id required")


        if not file_id or len(file_id)<1:
            raise tornado.web.HTTPError(400, "File id required")

        obj_serv = self.settings['object_service']

        preprocessor_list = obj_serv.get_operation_preprocessor(object_full_name, AccessType.DELETE)
        for preprocessor in preprocessor_list:
            yield preprocessor(object_full_name, None, AccessType.DELETE)

        yield obj_serv.delete_file(self.current_user, object_full_name, file_id)


        post_processor_list = obj_serv.get_operation_postprocessor(object_full_name, AccessType.DELETE)
        for post_processor in post_processor_list:
            yield  post_processor(obj_serv, object_full_name, None, AccessType.DELETE)

        self.write('{"result":"OK"}')
        self.finish()



"""
 Copyright (C) 2014 Maruf Maniruzzaman
 Website: http://cosmosframework.com
 Author: Maruf Maniruzzaman
 License :: OSI Approved :: MIT License
"""

from cosmos.service.utils import MongoObjectJSONEncoder
from cosmos.service import requesthandler
from cosmos.dataservice.objectservice import *


class SearchHandler(requesthandler.RequestHandler):

    @tornado.web.asynchronous
    @gen.coroutine
    def get(self, object_name):
        """
        Example:
        /search/cosmos.pages/?q=%22test%22&filter={%22type%22:%22page%22}&columns=title,createtime
        :param object_name:name of the object to search
        """

        #TODO: Unify with get service- just add the q parameter to the get

        q = self.get_argument("q", None, True)

        columns_str = self.get_argument("columns", None)
        filter_str = self.get_argument("filter", None)

        if filter_str:
            query = json.loads(filter_str)
        else:
            query = None

        if columns_str:
            columns = columns_str.split(',')
            columns = [column.strip() for column in columns]
        else:
            columns = []

        result_list = []

        if q:
            if query:
                query['$text'] = {'$search': q}
            else:
                query = {'$text': {'$search': q}}

            obj_serv = self.settings['object_service']

            cursor = obj_serv.text_search(self.current_user, object_name, query, columns)

            #TODO: use to_list to create list
            while(yield cursor.fetch_next):
                qry_result=cursor.next_object()
                result_list.append(qry_result)

        data = {"_d": MongoObjectJSONEncoder().encode(result_list), "_cosmos_service_array_result_": True}

        self.content_type = 'application/json'
        self.write(data)
        self.finish()




"""
 Copyright (C) 2014 Maruf Maniruzzaman
 Website: http://cosmosframework.com
 Author: Maruf Maniruzzaman
 License :: OSI Approved :: MIT License
"""

from cosmos.service import requesthandler

import tornado.ioloop
import tornado.web
import tornado.options
import tornado.template
import tornado.websocket
from tornado import gen

from cosmos.service.utils import MongoObjectJSONEncoder
from cosmos.dataservice.objectservice import *


class ServiceHandler(requesthandler.RequestHandler):
    def json_encode_result(self, result, is_list=False):
        if is_list:
            return {"_d": MongoObjectJSONEncoder().encode(result), "_cosmos_service_array_result_": True}
        else:
            return MongoObjectJSONEncoder().encode(result)

    @gen.coroutine
    def get(self, object_path):
        params = object_path.split('/')
        params = list(filter(len, params))

        if len(params) < 1 or len(params) > 2:
            raise tornado.web.HTTPError(404, "Not found")

        object_name = params[0]
        if not object_name:
            raise tornado.web.HTTPError(404, "Not found")

        id = None
        if len(params) == 2:
            id = params[1]

        obj_serv = self.settings['object_service']

        columns_str = self.get_argument("columns", None)
        filter_str = self.get_argument("filter", None)

        if filter_str:
            query = json.loads(filter_str)
        else:
            query = None

        if columns_str:
            columns = columns_str.split(',')
            columns = [column.strip() for column in columns]
        else:
            columns = []

        preprocessor_list = obj_serv.get_operation_preprocessor(object_name, AccessType.READ)
        for preprocessor in preprocessor_list:
            yield preprocessor(obj_serv, object_name, query, AccessType.READ)

        processor = None
        processor_list = obj_serv.get_operation_processor(object_name, AccessType.READ)
        assert isinstance(processor_list, list)

        if processor_list and len(processor_list) == 1:
            processor = processor_list[0]
        else:
            if len(processor_list) > 1:
                logging.critical("More than one READ processor found for object {}.".format(object_name))
                raise ValueError("More than one READ processor found for object {}.".format(object_name))

        result = None
        if id and len(id) > 0:
            if processor:
                cursor = processor(self.current_user, obj_serv, object_name, id, AccessType.READ, columns,
                                         find_one=True)
            else:
                cursor = obj_serv.load(self.current_user, object_name, id, columns)

            result = yield cursor
            if not result:
                raise tornado.web.HTTPError(404, "Not found")
            data = self.json_encode_result(result)
        else:
            if processor:
                cursor = processor(self.current_user, obj_serv, object_name, query, AccessType.READ, columns)
            else:
                cursor = obj_serv.find(self.current_user, object_name, query, columns)

            # TODO: use to_list to create list
            result_list = []
            while (yield cursor.fetch_next):
                qry_result = cursor.next_object()
                result_list.append(qry_result)
            result = result_list
            data = self.json_encode_result(result_list, True)

        post_processor_list = obj_serv.get_operation_postprocessor(object_name, AccessType.READ)
        for post_processor in post_processor_list:
            yield post_processor(obj_serv, object_name, result, AccessType.READ)

        self.content_type = 'application/json'
        self.write(data)
        self.finish()

    def clean_data(self, data):
        assert isinstance(data, dict)
        reserved_words = ["_id", "createtime", "modifytime", "owner", "g-recaptcha-response"]
        for word in reserved_words:
            if word in data:
                del data[word]

    @gen.coroutine
    def post(self, object_path):
        params = object_path.split('/')
        params = list(filter(len, params))
        object_name = params[0]
        try:
            data = json.loads(self.request.body.decode("utf-8"))
            assert isinstance(data, dict)
        except ValueError as ve:
            raise tornado.web.HTTPError(400, ve.message)

        # It is important that _id is not passed to save method, insert method is ok to use.
        # If _id is passed to save method it could overwrite object owned by other user
        self.clean_data(data)

        obj_serv = self.settings['object_service']

        preprocessor_list = obj_serv.get_operation_preprocessor(object_name, AccessType.INSERT)
        for preprocessor in preprocessor_list:
            yield preprocessor(obj_serv, object_name, data, AccessType.INSERT)

        processor = None
        processor_list = obj_serv.get_operation_processor(object_name, AccessType.INSERT)
        assert isinstance(processor_list, list)

        if processor_list and len(processor_list) == 1:
            processor = processor_list[0]
        else:
            if len(processor_list) > 1:
                logging.critical("More than one INSERT processor found for object {}.".format(object_name))
                raise ValueError("More than one INSERT processor found for object {}.".format(object_name))

        if processor:
            promise = processor(self.current_user, obj_serv, object_name, data, AccessType.INSERT)
        else:
            promise = obj_serv.insert(self.current_user, object_name, data)

        result = yield promise
        data = self.json_encode_result(result)

        post_processor_list = obj_serv.get_operation_postprocessor(object_name, AccessType.INSERT)
        for post_processor in post_processor_list:
            yield post_processor(obj_serv, object_name, result, AccessType.INSERT)

        self.write(data)
        self.finish()

    @gen.coroutine
    def put(self, object_path):
        params = object_path.split('/')
        params = list(filter(len, params))
        object_name = params[0]
        id = params[1]
        try:
            data = json.loads(self.request.body.decode("utf-8"))
            assert isinstance(data, dict)
        except ValueError as ve:
            raise tornado.web.HTTPError(400, ve.message)

        self.clean_data(data)

        obj_serv = self.settings['object_service']

        data['modifytime'] = str(datetime.datetime.now())

        preprocessor_list = obj_serv.get_operation_preprocessor(object_name, AccessType.UPDATE)

        for preprocessor in preprocessor_list:
            yield preprocessor(obj_serv, object_name, data, AccessType.UPDATE)

        processor = None
        processor_list = obj_serv.get_operation_processor(object_name, AccessType.UPDATE)
        assert isinstance(processor_list, list)

        if processor_list and len(processor_list) == 1:
            processor = processor_list[0]
        else:
            if len(processor_list) > 1:
                logging.critical("More than one UPDATE processor found for object {}.".format(object_name))
                raise ValueError("More than one UPDATE processor found for object {}.".format(object_name))

        if processor:
            promise = processor(self.current_user, obj_serv, object_name, data, AccessType.UPDATE, id=id)
        else:
            promise = obj_serv.update(self.current_user, object_name, id, data)

        result = yield promise
        data = MongoObjectJSONEncoder().encode(
            {"error": result.get("err"), "n": result.get("n"), "ok": result.get("ok"),
             "updatedExisting": result.get("updatedExisting")})

        post_processor_list = obj_serv.get_operation_postprocessor(object_name, AccessType.UPDATE)
        for post_processor in post_processor_list:
            yield post_processor(obj_serv, object_name, result, AccessType.UPDATE)

        self.write(data)
        self.finish()

    @gen.coroutine
    def delete(self, object_path):
        params = object_path.split('/')
        params = list(filter(len, params))
        object_name = params[0]
        id = params[1]

        obj_serv = self.settings['object_service']

        preprocessor_list = obj_serv.get_operation_preprocessor(object_name, AccessType.DELETE)
        for preprocessor in preprocessor_list:
            yield preprocessor(obj_serv, object_name, None, AccessType.DELETE)

        processor = None
        processor_list = obj_serv.get_operation_processor(object_name, AccessType.DELETE)
        assert isinstance(processor_list, list)

        if processor_list and len(processor_list) == 1:
            processor = processor_list[0]
        else:
            if len(processor_list) > 1:
                logging.critical("More than one DELETE processor found for object {}.".format(object_name))
                raise ValueError("More than one DELETE processor found for object {}.".format(object_name))

        if processor:
            promise = processor(self.current_user, obj_serv, object_name, None, AccessType.DELETE, id=id)
        else:
            promise = obj_serv.delete(self.current_user, object_name, id)
        result = yield promise

        data = MongoObjectJSONEncoder().encode(
            {"error": result.get("err"), "n": result.get("n"), "ok": result.get("ok")})

        post_processor_list = obj_serv.get_operation_postprocessor(object_name, AccessType.DELETE)
        for post_processor in post_processor_list:
            yield post_processor(obj_serv, object_name, result, AccessType.DELETE)

        self.write(data)
        self.finish()



"""
 Copyright (C) 2014 Maruf Maniruzzaman
 Website: http://cosmosframework.com
 Author: Maruf Maniruzzaman
 License :: OSI Approved :: MIT License
"""

import logging
from tornado.web import *
from cosmos.rbac.service import RbacService
from cosmos.service.constants import *
from cosmos.service.utils import MongoObjectJSONEncoder

class RequestHandler(tornado.web.RequestHandler):
    def __init__(self, *args, **kwargs):
        tornado.web.RequestHandler.__init__(self, *args, **kwargs)
        #self.rbac_service = RbacService()
        self.object_service = self.settings["object_service"]

    def check_access(self, user, object_name, properties, access):
        if user:
            username = tornado.escape.xhtml_escape(user.get("name", None))
        else:
            username = None

        role = self.rbac_service.get_roles(username)
        has_access = self.object_service.has_access(role, object_name, properties, access)

        if not has_access:
            raise tornado.web.HTTPError(401, "Unauthorized")

    def get_current_user(self):
        user_json = self.get_secure_cookie(USER_COOKIE_NAME)
        logging.debug(user_json)

        if user_json:
            return tornado.escape.json_decode(user_json)

        return None #{ "username": "Guest", "roles":[ANONYMOUS_USER_ROLE_SID]}

    def set_current_user(self, user):
        json_user = MongoObjectJSONEncoder().encode(user)
        self.set_secure_cookie(USER_COOKIE_NAME, json_user)
        encoded_json = base64.b64encode(json_user.encode("utf-8"))
        self.set_cookie(USER_PLAINTEXT_COOKIE_NAME, encoded_json)

    def logout_current_user(self):
        self.clear_cookie(USER_COOKIE_NAME)
        self.clear_cookie(USER_PLAINTEXT_COOKIE_NAME)

    def get_properties(self, data, namespace=None):
        properties = data.keys()
        child_props = []

        for prop in properties:
            prop_data = data.get(prop, None)

            if prop_data and isinstance(prop_data, dict):
                child_namespace = (namespace + "." + prop) if namespace else prop
                child_props = child_props + self.get_properties(prop_data, child_namespace)

        if child_props:
            properties = properties + child_props

        properties = [p if not namespace else namespace + "." + p for p in properties]

        return  properties


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(
    name='cosmos',
    version='0.01.082.00',
    packages=['cosmos', 'cosmos.common', 'cosmos.admin', 'cosmos.admin.samples', 'cosmos.datamonitor', 'cosmos.dataservice',
              'cosmos.rbac', 'cosmos.schema', 'cosmos.service', 'cosmos.processors', 'test'],
    url='http://cosmosframework.com',
    license='MIT License',
    author='Maruf Maniruzzaman',
    author_email='marufm@cosmosframework.com',
    description='Thin server application framework',

    classifiers=[
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Application Frameworks',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 2.7',

        'Programming Language :: Python :: 3.5',

        'Programming Language :: JavaScript'
    ],

    install_requires=['tornado', 'motor', 'mongolog', 'mock', 'requests', 'python-memcached', 'pycrypto', 'python_jwt', 'funcsigs', 'pbr'],
    # python 2 - pip install future

    entry_points = {
        'console_scripts': [
            'cosmos = cosmos.admin.commands:admin_main'
        ]
    },

    test_suite="test"
)
